/* Channel.java */

import java.util.Vector;

/**
 * @author Fosco
 * 
 * This class realizes the channel
 */
public class Channel extends Thread{
  private final static int MAX_EVENTS = 3;

  private Vector subscribers = new Vector();

  private String[] msgInQueue = new String[MAX_EVENTS];

  private int msgInQueueLength = 0;

  /**
   * Run method
   */
  public void run() {
	    while (true) {
	      try {
	        Thread.sleep(5000);
	        notifyMsg();
	      }
	      catch (Exception e) {
	        System.out
	            .println("" + this.getClass() + ", errore! Il seguente: " + e);
	        e.printStackTrace();
	      }

	    }
	  }

  
  public synchronized void addSubscriber(Subscriber s) {
    // if the subscriber is not registered, add it to the list
    if (!subscribers.contains(s))
      subscribers.add(s);
  }


  public synchronized void removeSubscriber(Subscriber s) {
    // if the subscriber exists, remove it from the list
    if (subscribers.contains(s))
      subscribers.removeElement(s);
  }

  /**
   * Public method to be used by the Editor for the publication of events.
   *  
   */
  public synchronized void put(String msg) {
	try {
      while (msgInQueueLength > (msgInQueue.length - 1)) {
	    wait();
	  }
	  msgInQueue[msgInQueueLength] = msg;
	  msgInQueueLength++;
	}
	catch (Exception e) {
	  System.out.println("Canale, si e' presentato il seguente errore: " + e);
	  e.printStackTrace();
	}
  }

  private synchronized void notifyMsg() {
    String msg;
    int tot = subscribers.size();

    for (int k = 0; k < msgInQueueLength; k++) {
	  msg = msgInQueue[k];
	  for (int i = 0; i < tot; i++) {
	    Subscriber subscriberIth = (Subscriber) subscribers
	        .elementAt(i);
        subscriberIth.push(msg);
	  }
	}
    // DEBUG
    System.out.println("Channel: Emptied the queue!");
	// reset the counter
	msgInQueueLength = 0;
	    
    /*
     * reactivates processes blocked on the put method.
     */
    notifyAll();
  }
  
 }