/* Sottoscrittore.java */

import java.util.Random;
import java.util.Vector;

/**
 * @author Fosco
 * 
 * This class realizes the subscriber
 */
public class Subscriber extends Thread {
  private Channel channel;

  private int identifier = -1;

  private Vector eventQueue = new Vector();

  public Subscriber(Channel c, int id) {
    super("S"+id);
    channel = c;
    identifier = id;
  }

  public void run() {
    try {
      sleep((int) (Math.random() * 3000) + 2000);

      System.out.println("\nSubscriber (S" + identifier
          + "): begins and subscribes...");

      channel.addSubscriber(this);

      Random r = new Random();

      int N = 2 + (r.nextInt(30));

      System.out.println("Subscriber S" + identifier
          + " is ready to receive " + N + " messages");

      for (int i = 0; i < N; i++) {
        String readMsg = read();
        System.out.println("S" + identifier + ": " + readMsg);
      }

      channel.removeSubscriber(this);

      System.out.println("\nS" + identifier
          + ": unsubscribes and terminates.");
    } catch (Exception e) {
      System.err.println("An error occurred, the following: " + e);
      e.printStackTrace(System.err);
    }

  }

  /**
   * This call blocks when the event queue is empty.
   * NOTE: the read method consumes the message from the event queue.
   * 
   * @return the received message, the string "-1" as EOF, and null if there
   *         were problems
   */
  private synchronized String read() throws InterruptedException {
    if (eventQueue.size() == 0) wait();
    String firstReceivedMessage = (String) eventQueue.lastElement();
    eventQueue.remove(eventQueue.size() - 1);
    return firstReceivedMessage;
  }

  public synchronized void push(String msg) {
    eventQueue.insertElementAt(msg, 0);
    notify();
  }

}